@@ -11,50 +11,119 @@ from .dial import start_app, quit_app, get_device_status, get_app_status
 from .websocket import (PROTOCOL_RAMP, RAMP_ENABLED, RAMP_STATE_UNKNOWN,
                         RAMP_STATE_PLAYING, RAMP_STATE_STOPPED,
                         create_websocket_client)
-from .error import ConnectionError, NoChromecastFoundError
+from .youtube import play_youtube_video, play_youtube_playlist
+from .error import (
+    PyChromecastError, NoChromecastFoundError,
+    MultipleChromecastsFoundError, ChromecastConnectionError)
 
 
-def play_youtube_video(video_id, host=None):
-    """ Starts the YouTube app if it is not running and plays
-        specified video. """
-
-    if not host:
-        host = _auto_select_chromecast()
-
-    start_app(host, APP_ID["YOUTUBE"], {"v": video_id})
-
-
-def play_youtube_playlist(playlist_id, host=None):
-    """ Starts the YouTube app if it is not running and plays
-        specified playlist. """
-
-    if not host:
-        host = _auto_select_chromecast()
-
-    start_app(host, APP_ID["YOUTUBE"],
-              {"listType": "playlist", "list": playlist_id})
-
-
-def _auto_select_chromecast():
+def _get_all_chromecasts():
     """
-    Discovers local Chromecasts and returns first one found.
-    Raises exception if none can be found.
+    Returns a list of all chromecasts on the network as PyChromecast
+    objects.
     """
-    ips = discover_chromecasts(1)
+    ips = discover_chromecasts()
+    cc_list = []
+    for ip_address in ips:
+        try:
+            cc_list.append(Chromecast(host=ip_address))
+        except ConnectionError:
+            pass
+    return cc_list
 
-    if ips:
-        return ips[0]
+
+def get_chromecasts(**filters):
+    """
+    Searches the network and returns a list of Chromecast objects.
+    Filter is a list of options to filter the chromecasts by.
+
+    ex: get_chromecasts(friendly_name="Living Room")
+
+    May return an empty list if no chromecasts were found matching
+    the filter criteria
+
+    Filters include DeviceStatus items:
+        friendly_name, model_name, manufacturer, api_version
+    Or AppStatus items:
+        app_id, description, state, service_url, service_protocols (list)
+    Or ip address:
+        ip
+    """
+    cc_list = set(_get_all_chromecasts())
+    excluded_cc = set()
+
+    if not filters:
+        return list(cc_list)
+
+    if 'ip' in filters:
+        for chromecast in cc_list:
+            if chromecast.host != filter['ip']:
+                excluded_cc.add(chromecast)
+        filters.pop('ip')
+
+    for key, val in filters.items():
+        for chromecast in cc_list:
+            for tup in [chromecast.device, chromecast.app]:
+                if hasattr(tup, key) and val != getattr(tup, key):
+                    excluded_cc.add(chromecast)
+
+    filtered_cc = cc_list - excluded_cc
+    return list(filtered_cc)
+
+
+def get_chromecasts_as_dict(**filters):
+    """
+    Returns a dictionary of chromecasts with the friendly name as
+    the key.  The value is the pychromecast object itself.
+    """
+    return {cc.device.friendly_name: cc for cc in get_chromecasts(**filters)}
+
+
+def get_chromecast(strict=False, **filters):
+    """
+    Same as get_chromecasts but only if filter matches exactly one
+    ChromeCast.
+
+    Returns a Chromecast matching exactly the fitler specified.
+
+    If strict, return one and only one chromecast
+    """
+
+    # If we have filters or are operating in strict mode we have to scan
+    # for all Chromecasts to ensure there is only 1 matching chromecast.
+    # If no filters given and not strict just use the first dicsovered one.
+    if filters or strict:
+        results = get_chromecasts(**filters)
     else:
-        raise NoChromecastFoundError("Unable to detect Chromecast")
+        results = [Chromecast(ip) for ip in discover_chromecasts(1)]
+
+    if len(results) > 1:
+        if strict:
+            raise MultipleChromecastsFoundError(
+                'More than one Chromecast was found specifying '
+                'the filter criteria: {}'.format(filters))
+        else:
+            return results[0]
+
+    elif not results:
+        if strict:
+            raise NoChromecastFoundError(
+                'No Chromecasts matching filter critera were found:'
+                ' {}'.format(filters))
+        else:
+            return None
+
+    else:
+        return results[0]
 
 
-class PyChromecast(object):
+class Chromecast(object):
     """ Class to interface with a ChromeCast. """
 
-    def __init__(self, host=None):
+    def __init__(self, host):
         self.logger = logging.getLogger(__name__)
 
-        self.host = host if host else _auto_select_chromecast()
+        self.host = host
 
         self.logger.info("Querying device status")
         self.device = get_device_status(self.host)
@@ -163,10 +232,15 @@ class PyChromecast(object):
                 self._refresh_timer.cancel()
 
             self._refresh_timer = threading.Timer(5, self.refresh)
+            self._refresh_timer.daemon = True
             self._refresh_timer.start()
 
-    def __str__(self):
-        return "PyChromecast({}, {}, {}, {}, api={}.{})".format(
+    def __repr__(self):
+        return "Chromecast({}, {}, {}, {}, api={}.{})".format(
             self.host, self.device.friendly_name, self.device.model_name,
             self.device.manufacturer, self.device.api_version[0],
             self.device.api_version[1])
+
+
+# For backwards compatibility with 0.4.4
+PyChromecast = Chromecast
